@@ -6,7 +6,7 @@
 #include "lib.h"
 #include "transfer.h"
 
-extern uint32 share_cache_divider; /* 0.4.27.c11 */
+extern uint32 share_cache_divider;
 
 #ifdef __cplusplus
 extern "C" {
