@@ -15,9 +15,9 @@ extern "C" {
   /* give this function the number of lines after this one that
    * shouldn't be separated over a page break */
   int cli_output_line(int, char *);
-  void cli_outline_nl(char *t); /* 0.4.27.c14 */
+  void cli_outline_nl(char *t);
   void cli_out_lines(char *t1, char *t2, char *t3, char *t4, char *t5,
-		     char *t6); /* 0.4.27.c14 */
+		     char *t6);
 
   char wait_key_callback(void (*a)(void));
   void raw_mode(int);
@@ -25,13 +25,6 @@ extern "C" {
   extern char *output_bold_on;
   extern char *output_special_off;
 
-  /* 0.4.27.c14 removed macro "cli_output_text" and corresponding (unused)
-     routine by the same name */
-
-#ifdef HAVE_TERMCAP_H
-#include <termcap.h>
-#endif
-
 #ifdef HAVE_LIBTERMCAP
 #  define tputs_x(x) tputs(x,1,putchar)
 #else
