@@ -38,11 +38,12 @@
 #endif
 
 #include "conf.h"
+#include "gnut.h" /* 0.4.28.c01 */
 #include "hash.h"
 #include "host.h"
 #include "http.h"
-#include "net.h"
 #include "list.h"
+#include "net.h"
 #include "threads.h"
 
 Gnut_Hash *host_list = 0;
@@ -56,13 +57,8 @@ host_entry * host_lru;
 int next_lrup;
 uint32 hlru_num;
 
-#ifndef PTHREADS_DRAFT4
- pthread_mutex_t host_mutex=PTHREAD_MUTEX_INITIALIZER;
-#else
- pthread_mutex_t host_mutex;
-#endif
+/* 0.4.28.c01 Moved mutex to gnut.c */
 
-/* 0.4.27.c21 */
 eqhd_vendor evendors[] = {
   {"xxxx", "Unknown"},    /* This one must be first */
 
@@ -190,14 +186,13 @@ int host_ok_for_neither(uchar *ip)
 
 uchar host_hash(void *a)
 {
-  uint32 crc; /* 0.4.27.c09 */
+  uint32 crc;
   host_entry *he;
   he = a;
 
-  /* 0.4.27.c09 (all 6 following lines) */
   crc = crc32_block(&(he->ip[0]), 4);
-  crc32_add8(&crc, he->port[0]);
-  crc32_add8(&crc, he->port[1]);
+  crc32_add8(&crc, he->port_le[0]);
+  crc32_add8(&crc, he->port_le[1]);
   crc ^= (crc >> 16);
   crc ^= (crc >> 8);
   return (crc & 0xff);
@@ -211,7 +206,7 @@ int host_compare(void *a, void *b)
   he2 = (host_entry *) b;
 
   if ((memcmp(he1->ip, he2->ip, 4) == 0)
-      && (memcmp(he1->port, he2->port, 2) == 0)) {
+      && (memcmp(he1->port_le, he2->port_le, 2) == 0)) {
     return 0;
   }
   return -1;
@@ -223,8 +218,8 @@ void clear_he(host_entry * he)
   he->ip[1] = 0;
   he->ip[2] = 0;
   he->ip[3] = 0;
-  he->port[0] = 0;
-  he->port[1] = 0;
+  he->port_le[0] = 0;
+  he->port_le[1] = 0;
   he->files = 0;
   he->kbytes = 0;
 }
@@ -233,7 +228,7 @@ int host_init()
 {
   int i;
 
-  gd_s(3, "host_init entering\n");
+  GD_S(3, "host_init entering\n");
   host_list = gnut_hash_new(host_hash, host_compare);
 
   host_lru = ymaloc(HLRUSIZE * sizeof(host_entry), 154);
@@ -299,19 +294,19 @@ void host_rmv_core(host_entry *h)
   fre_hen(&h, 497);
 }
 
-int host_remove(uchar ip[4], uchar port[2])
+int host_remove(uchar ip[4], uchar port_le[2])
 {
   host_entry he;
   host_entry *he2;
 
-  gd_s(1,"host.remove entering ");
-  gd_i(1, ip[0]); gd_s(1, "."); gd_i(1, ip[1]); gd_s(1, ".");
-  gd_i(1, ip[2]); gd_s(1, "."); gd_i(1, ip[3]); gd_s(1, "\n");
+  GD_S(1,"host.remove entering ");
+  GD_I(1, ip[0]); GD_S(1, "."); GD_I(1, ip[1]); GD_S(1, ".");
+  GD_I(1, ip[2]); GD_S(1, "."); GD_I(1, ip[3]); GD_S(1, "\n");
 
   pthread_mutex_lock(&host_mutex);
 
   memcpy(he.ip, ip, 4);
-  memcpy(he.port, port, 2);
+  memcpy(he.port_le, port_le, 2);
 
   dqi(0x0154);
   he2 = gnut_hash_find(host_list, &he);
@@ -319,7 +314,7 @@ int host_remove(uchar ip[4], uchar port[2])
     pthread_mutex_unlock(&host_mutex);
     /* This happens if the same host was connected twice and both
      * connections close */
-    gd_s(1, "host.remove  host not found!\n");
+    GD_S(1, "host.remove  host not found!\n");
     return -1;
   }
 
@@ -338,8 +333,23 @@ host_entry * host_add(host_entry *he)
 
   rv = he;
 
-  gd_s(3, "host.add entering host_list="); gd_p(3, host_list);
-  gd_s(3, " he="); gd_p(3, he); gd_s(3, "\n");
+  GD_S(3, "host.add entering host_list="); GD_P(3, host_list);
+  GD_S(3, " he="); GD_P(3, he); GD_S(3, "\n");
+
+  { /* 0.4.28.c11 Detect and eliminate bogus file counts */
+    uint32 avgsize;
+
+    avgsize = he->files;
+    if (avgsize) {
+      avgsize = (he->kbytes / avgsize) << 10;
+    }
+    if (avgsize < 10000) {
+      if (he->files > 1000) {
+	he->files = 0;
+	he->kbytes = 0;
+      }
+    }
+  }
 
   pthread_mutex_lock(&host_mutex);
 
@@ -389,7 +399,7 @@ host_entry * host_add(host_entry *he)
 
   pthread_mutex_unlock(&host_mutex);
 
-  gd_s(3,"host.add returning\n");
+  GD_S(3,"host.add returning\n");
 
   return(rv);
 }
@@ -405,7 +415,6 @@ int host_restore(char *fname)
   char *buf;
   int s_ip[4];
   int s_port;
-  uint16 p16;
   int errf;
   int good;
   uint32 s_files;
@@ -414,8 +423,8 @@ int host_restore(char *fname)
   uint32 sr;
   int nserv;
 
-  /* 0.4.27.c27 Since we restore the share upload rate history in this
-     routine, we might as well init the other EQHD (extended QueryHits
+  /* Since we restore the share upload rate history in this routine,
+     we might as well init the other EQHD (extended QueryHits
      descriptor) values here too. */
   gh_did_upload = 0;
   gh_did_receive = 0;
@@ -423,27 +432,27 @@ int host_restore(char *fname)
 
   errf = 0;
 
-  gd_s(2, "host.restore entering\n");
+  GD_S(2, "host.restore entering\n");
 
   fp = fopen(fname, "r");
 
   if (fp == 0) {
-    gd_s(1, "host.restore can't open: "); gd_s(1, fname); gd_s(1, "\n");
+    GD_S(1, "host.restore can't open: "); GD_S(1, fname); GD_S(1, "\n");
     return -1;
   }
 
   host_clear();
 
-  gd_s(3, "host.restore host_list="); gd_p(3, host_list); gd_s(3, "\n");
-  gd_s(2, "host.restore starting to read hosts...\n");
+  GD_S(3, "host.restore host_list="); GD_P(3, host_list); GD_S(3, "\n");
+  GD_S(2, "host.restore starting to read hosts...\n");
 
   n1 = n2 = 0;
   he = 0;
-  nserv = 0; /* 0.4.27.c27 */
-  while (fgets(bf, sizeof(bf), fp)) { /* 0.4.27.c27 */
-    buf = bf; /* 0.4.27.c27 */
+  nserv = 0;
+  while (fgets(bf, sizeof(bf), fp)) {
+    buf = bf;
 
-    /* 0.4.27.c27 Skip leading spaces */
+    /* Skip leading spaces */
     while(*buf == ' ') {
       buf++;
     }
@@ -459,17 +468,17 @@ int host_restore(char *fname)
     good = 0;
     s_files = 0;
     s_kbytes = 0;
-    if (buf[0] == '#') { /* 0.4.27.c27 */
+    if (buf[0] == '#') {
       /* Comment -- ignore */
-    } else if (sscanf(buf, "%i.%i.%i.%i:%i %u %u", /* 0.4.27.c11 */
+    } else if (sscanf(buf, "%i.%i.%i.%i:%i %u %u",
                &s_ip[0], &s_ip[1], &s_ip[2], &s_ip[3],
 	       &s_port, &s_files, &s_kbytes) >= 5) {
       good = 1;
-    } else if (sscanf(buf, "%i.%i.%i.%i %i %u %u", /* 0.4.27.c11 */
+    } else if (sscanf(buf, "%i.%i.%i.%i %i %u %u",
                &s_ip[0], &s_ip[1], &s_ip[2], &s_ip[3],
                &s_port, &s_files, &s_kbytes) >= 5) {
       good = 1;
-    } else if (sscanf(buf, "servrate %i", &sr) >= 1) { /* 0.4.27.c27 */
+    } else if (sscanf(buf, "servrate %i", &sr) >= 1) {
       if (nserv < 10) {
 	urate_history[nserv] = sr;
       }
@@ -482,6 +491,8 @@ int host_restore(char *fname)
     }
 
     if (good) {
+      uint32 avgsize;
+
       /* printf("got host %03d.%03d.%03d.%03d:%05d %5d %9d\n",
              s_ip[0], s_ip[1], s_ip[2], s_ip[3],
 	     s_port, s_files, s_kbytes); */
@@ -489,9 +500,21 @@ int host_restore(char *fname)
       he->ip[1] = s_ip[1];
       he->ip[2] = s_ip[2];
       he->ip[3] = s_ip[3];
-      p16 = s_port;
-      p16 = GUINT16_TO_LE(p16);
-      memcpy(he->port, &p16, 2);
+
+      PUT_LEU16(he->port_le, s_port, 38);
+
+      /* 0.4.28.c11 Detect and eliminate bogus file counts */
+      avgsize = s_files;
+      if (avgsize) {
+	avgsize = (s_kbytes / avgsize) << 10;
+      }
+      if (avgsize < 10000) {
+	if (s_files > 1000) {
+	  s_files = 0;
+	  s_kbytes = 0;
+	}
+      }
+
       he->files = s_files;
       he->kbytes = s_kbytes;
 
@@ -517,7 +540,7 @@ int host_restore(char *fname)
     fre_hen(&he, 496);
   }
 
-  update_urate(); /* 0.4.27.c27 */
+  update_urate();
 
   if (errf >= 4) {
     printf("There are %d additional bad line(s) in %s.\n", errf - 3, fname);
@@ -527,12 +550,12 @@ int host_restore(char *fname)
 
   printf("Got %i addresses from hosts file.\n", hosts_num);
 
-  gd_s(2,"host_restore returning\n");
+  GD_S(2,"host_restore returning\n");
   return 0;
 }
 
 int hs_n1, hs_n2;
-int hwl_ignore_lru; /* 0.4.27.c10 */
+int hwl_ignore_lru;
 
 int host_write_line(void *a, void *b)
 {
@@ -541,15 +564,14 @@ int host_write_line(void *a, void *b)
   uint16 p16;
   char buf[100];
   int l1;
-  uint32 crc; /* 0.4.27.c10 */
-  int i; /* 0.4.27.c10 */
-  int oktowrite; /* 0.4.27.c10 */
+  uint32 crc;
+  int i;
+  int oktowrite;
   uint32 avgsize;
 
   he = a;
   fp = b;
 
-  /* 0.4.27.c10 */
   oktowrite = 1;
   if (hwl_ignore_lru) {
     crc = crc32_block(he, 6);
@@ -560,9 +582,8 @@ int host_write_line(void *a, void *b)
     }
   }
 
-  if (oktowrite) { /* 0.4.27.c10 */
-    memcpy(&p16, he->port, 2);
-    p16 = GUINT16_FROM_LE(p16);
+  if (oktowrite) {
+    p16 = GET_LEU16(he->port_le, 21);
     sprintf(buf, "%i.%i.%i.%i %i", he->ip[0], he->ip[1], he->ip[2], he->ip[3],
 	    p16);
     l1 = strlen(buf);
@@ -575,7 +596,7 @@ int host_write_line(void *a, void *b)
     if (avgsize) {
       avgsize = (he->kbytes / avgsize) << 10;
     }
-    fprintf(fp, "\t%u\t%u\t%u\n", he->files, he->kbytes, avgsize); /* 0.4.27.c11 */
+    fprintf(fp, "\t%u\t%u\t%u\n", he->files, he->kbytes, avgsize);
 
     hs_n1++;
     if (hs_n1 >= 1000) {
@@ -610,42 +631,31 @@ int host_save(char *fname)
     return -1;
   }
 
-  gd_s(3, "host_save opened file\n");
+  GD_S(3, "host_save opened file\n");
 
-  /* 0.4.27.c27 Since I'm changing the hosts file format I might as well
-     add comments to the host file explaining the format */
+  /* Print comments in the host file explaining the file's format */
   fprintf(fp, "# Hosts file created by gnut %s\n# Also contains other important info needed when restarting.\n#\n# The first 10 lines are a record of the speed (in bytes per second)\n# of the 10 most recent successful uploads.\n", VERSION);
 
-  /* 0.4.27.c27 Save the upload rates history */
+  /* Save the upload rates history */
   for(i=0; i<10; i++) {
     fprintf(fp, "servrate %i\n", urate_history[i]);
   }
 
   fprintf(fp, "#\n# The rest of this file contains hosts and their stats.\n# The 'num' column gives the number of shared files on that host, and the\n# 'size/1024' column gives the total size of those files divided by 1024.\n# If the sizes seem too big, remember that most shared files are about 5 to\n# 10 million bytes. The 'avg. size' column shows this average (in bytes).\n#\n# IP and port          num   size/1024 avg. size\n");
 
-  hwl_ignore_lru = 0; /* 0.4.27.c10 */
+  hwl_ignore_lru = 0;
   hs_n1 = hs_n2 = 0;
   for(i=0; i<HLRUSIZE; i++) {
     he = host_lru + i;
     if (he->ip[0]) {
       host_write_line(he, fp);
     }
-
-#if 0
-    if (he->ip[0]) {
-      uint16 p16;
-      memcpy(&p16, he->port, 2);
-      p16 = GUINT16_FROM_LE(p16);
-      fprintf(fp, "%i.%i.%i.%i %i\n", he->ip[0], he->ip[1],
-	      he->ip[2], he->ip[3], p16);
-    }
-#endif
   }
 
   pthread_mutex_lock(&host_mutex);
-  gd_s(3, "host_save locked mutex\n");
+  GD_S(3, "host_save locked mutex\n");
 
-  hwl_ignore_lru = 1; /* 0.4.27.c10 */
+  hwl_ignore_lru = 1;
   gnut_hash_foreach(host_list, host_write_line, fp);
 
   if (hs_n2) {
@@ -660,9 +670,9 @@ int host_save(char *fname)
 
 int host_totals(uint *num, float64 *files, float64 *size)
 {
-  gd_s(2, "host.totals num="); gd_i(2, hosts_num); gd_s(2, " files=");
-  gd_f64(2, hosts_files); gd_s(2, " size=");
-  gd_f64(2, (hosts_kbytes * 1024.0)); gd_s(2, "\n");
+  GD_S(2, "host.totals num="); GD_I(2, hosts_num); GD_S(2, " files=");
+  GD_F64(2, hosts_files); GD_S(2, " size=");
+  GD_F64(2, (hosts_kbytes * 1024.0)); GD_S(2, "\n");
 
   if (num) {
     *num = hosts_num;
@@ -681,20 +691,20 @@ int host_totals(uint *num, float64 *files, float64 *size)
   return 0;
 }
 
-int host_find(uchar ip[4], uchar port[2])
+int host_find(uchar ip[4], uchar port_le[2])
 {
   host_entry he,*ret;
 
   dqi(0x0139);
 
-  gd_s(3,"host_find entering "); gd_i(3, ip[0]); gd_s(3, ".");
-  gd_i(3, ip[1]); gd_s(3, "."); gd_i(3, ip[2]); gd_s(3, "."); gd_i(3, ip[3]);
-  gd_s(3, "\n");
+  GD_S(3,"host_find entering "); GD_I(3, ip[0]); GD_S(3, ".");
+  GD_I(3, ip[1]); GD_S(3, "."); GD_I(3, ip[2]); GD_S(3, "."); GD_I(3, ip[3]);
+  GD_S(3, "\n");
 
   pthread_mutex_lock(&host_mutex);
 
   memcpy(he.ip, ip, 4);
-  memcpy(he.port, port, 2);
+  memcpy(he.port_le, port_le, 2);
 
   dqi(0x0156);
   ret = gnut_hash_find(host_list,&he);
@@ -702,9 +712,9 @@ int host_find(uchar ip[4], uchar port[2])
   pthread_mutex_unlock(&host_mutex);
 
   if (ret) {
-    gd_s(2,"found ip "); gd_i(2, ret->ip[0]); gd_s(2, ".");
-    gd_i(2, ret->ip[1]); gd_s(2, "."); gd_i(2, ret->ip[2]); gd_s(2, ".");
-    gd_i(2, ret->ip[3]); gd_s(2, "\n");
+    GD_S(2,"found ip "); GD_I(2, ret->ip[0]); GD_S(2, ".");
+    GD_I(2, ret->ip[1]); GD_S(2, "."); GD_I(2, ret->ip[2]); GD_S(2, ".");
+    GD_I(2, ret->ip[3]); GD_S(2, "\n");
     return 0;
   }
   dqi(0x013a);
@@ -715,12 +725,12 @@ int host_clear()
 {
   pthread_mutex_lock(&host_mutex);
 
-  gd_s(3, "host_clear entering\n");
+  GD_S(3, "host_clear entering\n");
   gnut_hash_fre(host_list);
 
   host_list = gnut_hash_new(host_hash, host_compare);
 
-  gd_s(3, "host_clear new host_list="); gd_p(3, host_list); gd_s(3, "\n");
+  GD_S(3, "host_clear new host_list="); GD_P(3, host_list); GD_S(3, "\n");
 
   hosts_num = 0;
   hosts_files = 0;
