@@ -15,29 +15,12 @@
 #define VPN_P    0x08
 #define BAD_ADDR 0x80
 
-typedef struct {
-  char * vcode;
-  char * appname;
-} eqhd_vendor; /* 0.4.27.c21 */
-
-typedef struct {
-  uchar ip[4];
-  uchar port[2];
-  uint32  files;
-  uint32  kbytes;
-} host_entry;
-
-typedef union {
-  struct in_addr a;
-  uchar          b[sizeof(struct in_addr)];
-} ip_t; /* to ensure correct alignment on architectures that require it */
-
 #ifdef __cplusplus
 extern "C" {
 #endif
 
-  extern eqhd_vendor evendors[]; /* 0.4.27.c21 */
-  extern uint32 hlru_num; /* 0.4.27.c25 */
+  extern eqhd_vendor evendors[];
+  extern uint32 hlru_num;
 
   void fre_hen(host_entry **x, int bugnum);
 
