@@ -3,21 +3,17 @@
 
    This file is distributed under the GPL, see file COPYING for details */
 
-
+#include <errno.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <stdarg.h>
 #include <string.h>
 #include <pthread.h>
 #include <ctype.h>
-#ifndef WIN32
 #include <unistd.h>
-#else
-#include <io.h>
-#endif
 
 #ifdef MCHECK
-#include <mcheck.h>
+# include <mcheck.h>
 #endif
 
 #include "conf.h"
@@ -176,6 +172,9 @@ void dq_dump(char *msg)
 
 /* ----------------------------- memory management and leak detection ------ */
 
+/* 20010922 Leaks to investigate (in order of liklihood): 486, 285,
+   506, 455, 508 */
+
 int _gnut_mprobe(void *d)
 {
 #ifdef MCHECK
@@ -187,6 +186,7 @@ int _gnut_mprobe(void *d)
 
 #define LEAKIDX 700
 long leakers[LEAKIDX];
+long leak_hi[LEAKIDX];
 long leak_d[LEAKIDX];
 long reset_count;
 long reset_thres;
@@ -198,9 +198,11 @@ void init_leaks()
 
   for(i=0; i<LEAKIDX; i++) {
     leakers[i] = 0;
+    leak_hi[i] = 0;
     leak_d[i] = 0;
   }
   leakers[1] = 1;
+  leak_hi[1] = 1;
   reset_count = 0;
   reset_thres = 100;
 }
@@ -220,17 +222,24 @@ void reset_leaks()
 void tell_leaks()
 {
   int i;
-  int max, maxi, maxd;
+  int maxh, maxc, maxi, maxd;
 
-  max = 0; maxi = 0; maxd = 0;
+  maxh = 0; maxc = 0; maxi = 0; maxd = 0;
   for(i=0; i<LEAKIDX; i++) {
     if (leak_d[i] > maxd) {
       maxd = leak_d[i];
-      max = leakers[i];
+      maxh = leak_hi[i];
+      maxc = leakers[i];
       maxi = i;
     }
   }
-  printf("yma""loc(..., %d) == %d, delta %d\n", maxi, max, maxd);
+  if (maxi) {
+    printf("yma""loc(..., %d) == %d, hi %d, delta %d\n", maxi, maxc, maxh, maxd);
+  } else {
+    printf("No new yma""loc record setter\n");
+  }
+
+  reset_leaks();
 }
 
 /* Every time a block is allocated via yma.loc, this routine is called.
@@ -278,11 +287,14 @@ int yprobe(void *a, int tracking_number)
 
   if (a) {
     dqi(0x017f);
+    dqi(0x8000 + tracking_number);
     /* Calculate positions of end tag blocks */
     p2 = (ymheader *) (((uint8 *) (a)) - sizeof(ymheader));
     usiz = p2->usize;
     p3 = (ymfooter *) (((uint8 *) (a)) + usiz);
 
+    dqi(0x0184);
+
     /* Check canaries */
     tn1 = p2->tn;
     if (p2->canarylo != (tn1 ^ 0x55555555)) {
@@ -314,7 +326,7 @@ void yfre(void **a, int tracking_number)
   int ypr;
 #endif
 
-  gd_s(5, "yfre a="); gd_p(5, a); gd_s(5, " *a="); gd_p(5, *a); gd_s(5, "\n");
+  GD_S(5, "yfre a="); GD_P(5, a); GD_S(5, " *a="); GD_P(5, *a); GD_S(5, "\n");
   if (*a == 0) {
     if (tracking_number) {
       printf("Double-fr""ee error, case %d\n", tracking_number);
@@ -325,12 +337,13 @@ void yfre(void **a, int tracking_number)
   }
 
 #if YTAGS
-  ypr = YPROBE(*a, tracking_number)
+  ypr = YPROBE(*a, tracking_number);
   if (ypr) {
     p2 = (ymheader *) (((uint8 *) (*a)) - sizeof(ymheader));
     tn1 = p2->tn;
-    leakers[tn1]--;
-    leak_d[tn1]--;
+    if (tn1 < LEAKIDX) {
+      leakers[tn1]--;
+    }
     free((void *) p2);
   }
 
@@ -398,10 +411,15 @@ void *ymaloc(int size, int tracking_number)
   p3->canaryhi = (tracking_number ^ 0xaaaaaaaa);
 #endif
 
-  leakers[tracking_number]++;
-  leak_d[tracking_number]++;
+  if (tracking_number < LEAKIDX) {
+    leakers[tracking_number]++;
+    if (leak_hi[tracking_number] < leakers[tracking_number]) {
+      leak_hi[tracking_number] = leakers[tracking_number];
+      leak_d[tracking_number]++;
+    }
+  }
 
-  check_reset();
+  /* check_reset(); */
 
   /*  printf("ymaloc src %d size %d\n", tracking_number, size + tsiz); */
 
@@ -458,41 +476,288 @@ char *path_slash = "/";
 
 /* -------------------- character type, case mapping, 8th bit, etc. ------- */
 
-/* isblank is a GNU library extension but I can only use POSIX and ANSI C */
-int g_isblank(char c) /* 0.4.27.c06 */
+/* isblank is a GNU library extension but gnut can only use POSIX and
+   ANSI C due to all the non-POSIX/ANSI platforms our users compile it
+   on */
+int g_isblank(int c) /* 0.4.28.c18 */
 {
   return((c == ' ') || (c == '\t'));
 }
 
 /* ----------------------------- varstring -------------------------------- */
 
-void fre_vs(varstring **x, int bugnum)
+/* 0.4.28.c06 Changed design of these routines (they weren't even
+   being used yet) */
+
+/* These aren't used yet, they're here because I plan to clean up the
+   http.c code; currently it has fixed buffer sizes (mostly 512. bytes)
+   and no checks */
+
+/* Here's an example of a varstring allocated in memory:
+
+                   P-.
+                     v
+   +-------+-------+---+---+---+---+---+-       -+---+---+
+   |alloc  |used   | H | e | l | l | o |  . . .  | d |NUL|
+   +-------+-------+---+---+---+---+---+-       -+---+---+
+    ^
+   V'
+
+   The 'char *' used as a parameter and return value to the vasrstring
+   routines is the pointer P in the diagram, P is actually a pointer
+   to the 8th byte of a malloc block. There are two header fields that
+   specify how much space has been allocated for the string and how
+   much it's actually using. The string gets reallocated when
+   necessary and reallocation happens in exponentially-increasing
+   chunks so as to provide a good tradeoff between time efficiency and
+   memory efficiency.
+
+   For most operations (including all non-length-modifying operations
+   like printing, strlen, tolower, etc.) you can pretend that your
+   varstring pointer is just a normal char *.
+*/
+
+void fre_vs(varstring_data **x, int bugnum)
 {
   yfre((void **) x, bugnum);
 }
 
-varstring * vs_new(void)
+char * vs_new(void)
 {
-  varstring * rv;
+  varstring_data * vs;
+  int ndata;
+  char * rv;
 
-  rv = ymaloc(sizeof(varstring), 541);
-  rv->vs_alloc = 10;
-  rv->vs_data = ymaloc(rv->vs_alloc, 542);
-  rv->vs_used = 0;
+  ndata = 10;
+
+#if YTAGS
+  vs = ymaloc(sizeof(varstring_alloc) + ndata, 541);
+#else
+  vs = malloc(sizeof(varstring_alloc) + ndata);
+#endif
+
+  vs->vs_alloc = ndata;
+  vs->vs_used = 0;
+
+  rv = &(vs->vs_data[0]);
 
   return rv;
 }
 
-void vs_kill(varstring *vs)
+void vs_free(char ** p)
 {
-  if(vs) {
-    if(vs->vs_data) {
-      fre_str(&(vs->vs_data), 543);
+  varstring_data * vs;
+
+  if (p) {
+    if (*p) {
+      vs = (varstring_data *) ((*p) - sizeof(varstring_alloc));
+
+#if YTAGS
+      fre_vs(&vs, 544);
+#else
+      free(vs);
+#endif
+      *p = 0;
     }
-    fre_vs(&vs, 544);
   }
 }
 
+/* ----------------------------- endianness ------------------------------- */
+
+/* 0.4.28.c08 Add all these endian conversion routines. Note, these routines
+   are not actually used unless YTAGS is defined. When YTAGS is not defined
+   (which is the normal case) endian conversion is done with inline macros */
+
+#ifdef TEST_BE2
+int tbe1_count = 0;
+uint32 tbe2_count = 0;
+uint32 tbe2_lim = 1;
+
+void tbe2(char *s, uint32 i1, int tn);
+void tbe2(char *s, uint32 i1, int tn)
+{
+  if (tn) {
+    tbe1_count++;
+    if (tbe1_count < TEST_BE2) {
+      if (s) {
+	printf("%s", s);
+      }
+      if (i1) {
+	printf("%u ", i1);
+      }
+    }
+
+    tbe2_count++;
+    if (tbe2_count > tbe2_lim) {
+      tbe2_lim *= 2;
+      tbe1_count = 0;
+      printf("\ntbe2 %u\n", tbe2_count);
+    }
+  }
+}
+#else
+# define tbe2(s,i1,tn) while(0) { }
+#endif
+
+#if YTAGS
+
+/* If YTAGS is defined, these routines are used in place of the
+   inline macros in order to force type checking. */
+
+/* Code for running on a bigendian machine and getting from
+   littleendian format */
+uint16 be_get_leu16(uint8 * x, int tn)
+{
+  uint16 rv;
+
+  tbe2("be.get.leu16 ", tn, tn);
+  rv = *x;
+  rv |= ((uint16) (x[1])) << 8;
+  tbe2(0,rv, tn); tbe2("\n",0, tn);
+  return rv;
+}
+
+/* Code for running on a littleendian machine and getting from
+   littleendian format */
+uint16 le_get_leu16(uint8 * x, int tn)
+{
+  uint16 rv;
+
+  tbe2("le.get.leu16 ", tn, tn);
+
+  rv = *((uint16 *) x);
+
+#ifdef TEST_BE1
+  {
+    uint16 r2;
+    uint8 * p;
+
+    r2 = rv;
+    p = ((uint8 *) (&r2));
+    rv = *p;
+    rv |= ((uint16) (p[1])) << 8;
+  }
+#endif
+
+  tbe2(0, rv, tn); tbe2("\n", 0, tn);
+
+  return rv;
+}
+
+/* Code for running on a bigendian machine and storing into
+   littleendian format */
+void be_put_leu16(uint8 * p, uint16 x, int tn)
+{
+  tbe2("be.put.leu16 ", tn, tn);
+  tbe2(0, x, tn); tbe2("\n", 0, tn);
+
+  *p = x & 0xff;
+  x >>= 8;
+  p[1] = x;
+}
+
+/* Code for running on a littleendian machine and storing into
+   littleendian format */
+void le_put_leu16(uint8 * p, uint16 x, int tn)
+{
+  tbe2("le.put.leu16 ", tn, tn);
+  tbe2(0, x, tn); tbe2("\n", 0, tn);
+
+#ifdef TEST_BE1
+  *p = x & 0xff;
+  x >>= 8;
+  p[1] = x;
+#else
+  *((uint16 *) p) = x;
+#endif
+}
+
+/* Code for running on a bigendian machine and getting from
+   littleendian format */
+uint32 be_get_leu32(uint8 * x, int tn)
+{
+  uint32 rv;
+
+  tbe2("le.get.leu32 ", tn, tn);
+
+  rv = *x;
+  rv |= ((uint32) (x[1])) << 8;
+  rv |= ((uint32) (x[2])) << 16;
+  rv |= ((uint32) (x[3])) << 24;
+
+  tbe2(0, rv, tn); tbe2("\n", 0, tn);
+
+  return rv;
+}
+
+/* Code for running on a littleendian machine and getting from
+   littleendian format */
+uint32 le_get_leu32(uint8 * x, int tn)
+{
+  uint32 rv;
+
+  tbe2("le.get.leu32 ", tn, tn);
+
+  rv = *((uint32 *) x);
+
+#ifdef TEST_BE1
+  {
+    uint32 r2;
+    uint8 * p;
+
+    r2 = rv;
+    p = ((uint8 *) (&r2));
+
+    rv = *p;
+    rv |= ((uint32) (p[1])) << 8;
+    rv |= ((uint32) (p[2])) << 16;
+    rv |= ((uint32) (p[3])) << 24;
+  }
+#endif
+
+  tbe2(0, rv, tn); tbe2("\n", 0, tn);
+
+  return rv;
+}
+
+/* Code for running on a bigendian machine and storing into
+   littleendian format */
+void be_put_leu32(uint8 * p, uint32 x, int tn)
+{
+  tbe2("be.put.leu32 ", tn, tn);
+  tbe2(0, x, tn); tbe2("\n", 0, tn);
+
+  *p = x & 0xff;
+  x >>= 8;
+  p[1] = x & 0xff;
+  x >>= 8;
+  p[2] = x & 0xff;
+  x >>= 8;
+  p[3] = x;
+}
+
+/* Code for running on a littleendian machine and storing into
+   littleendian format */
+void le_put_leu32(uint8 * p, uint32 x, int tn)
+{
+  tbe2("le.put.leu32 ", tn, tn);
+  tbe2(0, x, tn); tbe2("\n", 0, tn);
+
+#ifdef TEST_BE1
+  *p = x & 0xff;
+  x >>= 8;
+  p[1] = x & 0xff;
+  x >>= 8;
+  p[2] = x & 0xff;
+  x >>= 8;
+  p[3] = x;
+#else
+  *((uint32 *) p) = x;
+#endif
+}
+
+#endif /* YTAGS */
+
 /* ----------------------------- CRC -------------------------------------- */
 /* CRC-16
 
@@ -544,7 +809,7 @@ void crc32_start(uint32 *crc)
 {
   int i, j;
   uint32 cctemp;
-  
+
   *crc = 0xffffffffL;
   if (crc32_init == 0) {
     for(i=0; i<256; i++) {
@@ -565,7 +830,7 @@ void crc32_start(uint32 *crc)
 void crc32_add8(uint32 *crc, uchar data)
 {
   uchar index;
-  
+
   index = (*crc >> 24L) ^ data;
   *crc = (*crc << 8L) ^ crc32_conv_8[index];
 }
@@ -613,6 +878,11 @@ uint32 crc32_block(void * data, uint32 len)
   pthread_mutex_t mutex_mutex;
 #endif
 
+#if 0
+/* This is disabled right now because I the mutex problems disappeared
+   before I started to use this routine, but if you're having the
+   mutex problem described here you might want to use this code */
+
 /* Because of unsolved bugs in gnut, I need to be able to clean up after
    dead threads, including possibly clearing a mutex that is locked by a
    dead thread. Since pthreads doesn't guarantee that a mutex can be cleared
@@ -647,6 +917,9 @@ int pmutex_lock(int *m)
   }
   return rv;
 }
+#endif
+
+/* ----------------------------- strings ---------------------------------- */
 
 char *expand_path(char *a)
 {
@@ -660,7 +933,7 @@ char *expand_path(char *a)
 	/*    GetWindowsDirectory(c, MAX_PATH); */
     /* I did this because too many people couldn't find
      * the .gnut* files under win32  -jp */
-    c[0] = 0;  
+    c[0] = 0;
 #endif
     b = (char *)ymaloc(strlen(a)+strlen(c)+3, 321);
     b[0] = 0;
@@ -680,8 +953,8 @@ char *expand_path(char *a)
     }
   }
 
-  return b;   
-}                                               
+  return b;
+}
 
 /* Counts the alphanumeric characters in a string. */
 int strlen_an(char *s)
@@ -689,7 +962,7 @@ int strlen_an(char *s)
   int rv;
 
   for(rv=0; *s; s++) {
-    if (isalnum(*s)) { /* 0.4.27.c07 */
+    if (G_ISALNUM(*s)) {
       rv++;
     }
   }
@@ -700,7 +973,7 @@ int strlen_an(char *s)
 void print_ascii(uchar *s)
 {
   while(*s) {
-    if (isprint(*s)) {
+    if (G_ISPRINT(*s)) {
       putchar(*s);
     } else {
       putchar('#');
@@ -715,7 +988,7 @@ void print_asc_n(uchar *s, int len)
   int i;
 
   for(i=0; i<len; i++) {
-    if (isprint(s[i])) {
+    if (G_ISPRINT(s[i])) {
       putchar(s[i]);
     } else {
       putchar('#');
@@ -764,7 +1037,7 @@ void make_lc(char *str)
   char *s;
 
   for(s=str; *s; s++) {
-    *s = tolower(*s); /* 0.4.27.c07 */
+    *s = tolower(*s);
   }
 }
 
@@ -784,7 +1057,34 @@ void make_7bit(char *str)
   }
 }
 
-/* 0.4.28.c15 */
+/* 0.4.28.c15
+
+   make_htmlsafe Removes characters that might be dangerous if the
+   string were to be printed into an HTML page.
+
+   This is mainly used for security reasons by http.c. For example, if
+   someone shared a file called foo.mp3, and someone else accesses gnut
+   through the HTML interface, gnut will send the HTML user a page that
+   includes something like this:
+
+   <a href="http://1.2.3.4:56/get/1/foo.mp3"> foo.mp3 </a>
+
+   Now imagine what would happen if the filename was:
+
+      xxx<hr>yyy.mp3
+
+   then gnut would put this on its HTML listing:
+
+   <a href="http://1.2.3.4:56/get/1/xxx<hr>yyy.mp3"> xxx<hr>yyy.mp3 </a>
+
+   and the HTML user would see a <hr> (horizontal line) in the middle
+   of the filename link. In theory this could be abused (although it's
+   hard because gnut doesn't display anything before a '/', and
+   therefore any tags like </b> and urls like http://myporn.com will
+   get clipped).
+
+   Anyway, to prevent this, all filenames are filtered through this
+   routine before being displayed in a web page. */
 void make_htmlsafe(char *str)
 {
   char *s;
@@ -800,7 +1100,6 @@ void make_htmlsafe(char *str)
   }
 }
 
-/* 0.4.27.c14 */
 /* Like strncmp, but also has an ignore-case option */
 int gl_strncmp(char *a, char* b, int len, int igcase)
 {
@@ -835,7 +1134,6 @@ int gl_strncmp(char *a, char* b, int len, int igcase)
   return rv;
 }
 
-/* 0.4.27.c11 */
 /* Convert a string to a 32-bit UNSIGNED quantity, like strtol. atol is
    similar, but the manpage doesn't say whether it handles bith signed
    and unsigned (it could if they wrote it right). I don't want to take
@@ -854,13 +1152,13 @@ uint32 gl_stou32(char *s, char **p)
   s1 = s;
   gotone = 0;
 
-  while(isspace(*s)) {
+  while(G_ISSPACE(*s)) {
     s++;
   }
   if (*s == '+') {
     s++;
   }
-  while(isdigit(*s)) {
+  while(G_ISDIGIT(*s)) {
     uint32 d;
 
     gotone = 1;
@@ -880,7 +1178,12 @@ uint32 gl_stou32(char *s, char **p)
 
 /* ----------------------------- debug messages and logging --------------- */
 
-int gnut_lib_debug=0;
+#if GDEBUG_ENABLE
+
+int gnut_lib_debug = 0;
+
+#define GDFLUSH_O fflush(stdout)
+#define GDFLUSH_E fflush(stderr)
 
 /* These gd_xxx routines are for debug printf's. There is one for each
    type of thing you might want to print, like strings, hex bytes,
@@ -889,153 +1192,134 @@ int gnut_lib_debug=0;
    failure to pass the right parameters on the stack. I eliminated
    those bugs by removing all use of var_args routines including
    printf */
-void gd_s(int level, char * str)
+void gl_gd_s(int level, char * str)
 {
   if (gnut_lib_debug >= level) {
     pthread_mutex_lock(&_g_debug_mutex);
-#ifndef WIN32
-    fflush(stdout);
-#endif
+    GDFLUSH_O;
     fprintf(stderr, "%s", str);
-#ifndef WIN32
-    fflush(stderr);
-#endif
+    GDFLUSH_E;
     pthread_mutex_unlock(&_g_debug_mutex);
   }
 }
 
-void gd_02x(int level, uchar c)
+void gl_gd_02x(int level, uchar c)
 {
   if (gnut_lib_debug >= level) {
     pthread_mutex_lock(&_g_debug_mutex);
-#ifndef WIN32
-    fflush(stdout);
-#endif
+    GDFLUSH_O;
     fprintf(stderr, "%02x", c);
-#ifndef WIN32
-    fflush(stderr);
-#endif
+    GDFLUSH_E;
     pthread_mutex_unlock(&_g_debug_mutex);
   }
 }
 
-void gd_i(int level, int32 x)
+void gl_gd_i(int level, int32 x)
 {
   if (gnut_lib_debug >= level) {
     pthread_mutex_lock(&_g_debug_mutex);
-#ifndef WIN32
-    fflush(stdout);
-#endif
+    GDFLUSH_O;
     fprintf(stderr, "%i", x);
-#ifndef WIN32
-    fflush(stderr);
-#endif
+    GDFLUSH_E;
     pthread_mutex_unlock(&_g_debug_mutex);
   }
 }
 
-/* 0.4.27.c11 Note, I did not flag the calls to gd_i that got changed to gd_u,
-   you can find them just by searching the code for "gd_u" */
-void gd_u(int level, uint32 x)
+void gl_gd_u(int level, uint32 x)
 {
   if (gnut_lib_debug >= level) {
     pthread_mutex_lock(&_g_debug_mutex);
-#ifndef WIN32
-    fflush(stdout);
-#endif
+    GDFLUSH_O;
     fprintf(stderr, "%u", x);
-#ifndef WIN32
-    fflush(stderr);
-#endif
+    GDFLUSH_E;
     pthread_mutex_unlock(&_g_debug_mutex);
   }
 }
 
-void gd_x(int level, int32 x)
+void gl_gd_x(int level, int32 x)
 {
   if (gnut_lib_debug >= level) {
     pthread_mutex_lock(&_g_debug_mutex);
-#ifndef WIN32
-    fflush(stdout);
-#endif
+    GDFLUSH_O;
     fprintf(stderr, "%x", x);
-#ifndef WIN32
-    fflush(stderr);
-#endif
+    GDFLUSH_E;
     pthread_mutex_unlock(&_g_debug_mutex);
   }
 }
 
-void gd_li(int level, int32 x)
+void gl_gd_li(int level, int32 x)
 {
   if (gnut_lib_debug >= level) {
     pthread_mutex_lock(&_g_debug_mutex);
-#ifndef WIN32
-    fflush(stdout);
-#endif
+    GDFLUSH_O;
     fprintf(stderr, "%li", (long int) x);
-#ifndef WIN32
-    fflush(stderr);
-#endif
+    GDFLUSH_E;
     pthread_mutex_unlock(&_g_debug_mutex);
   }
 }
 
-void gd_lu(int level, uint32 x)
+void gl_gd_lu(int level, uint32 x)
 {
   if (gnut_lib_debug >= level) {
     pthread_mutex_lock(&_g_debug_mutex);
-#ifndef WIN32
-    fflush(stdout);
-#endif
+    GDFLUSH_O;
     fprintf(stderr, "%lu", (long unsigned int) x);
-#ifndef WIN32
-    fflush(stderr);
-#endif
+    GDFLUSH_E;
     pthread_mutex_unlock(&_g_debug_mutex);
   }
 }
 
-void gd_f64(int level, float64 x)
+void gl_gd_f64(int level, float64 x)
 {
   if (gnut_lib_debug >= level) {
     pthread_mutex_lock(&_g_debug_mutex);
-#ifndef WIN32
-    fflush(stdout);
-#endif
+    GDFLUSH_O;
     fprintf(stderr, "%g", x);
-#ifndef WIN32
-    fflush(stderr);
-#endif
+    GDFLUSH_E;
     pthread_mutex_unlock(&_g_debug_mutex);
   }
 }
 
-void gd_p(int level, void *p)
+void gl_gd_p(int level, void *p)
 {
   if (gnut_lib_debug >= level) {
     pthread_mutex_lock(&_g_debug_mutex);
-#ifndef WIN32
-    fflush(stdout);
-#endif
+    GDFLUSH_O;
     fprintf(stderr, "%p", p);
-#ifndef WIN32
-    fflush(stderr);
-#endif
+    GDFLUSH_E;
     pthread_mutex_unlock(&_g_debug_mutex);
   }
 }
 
+void gl_gd_perror(int level, char * s)
+{
+#if GDEBUG_ENABLE
+  int er;
+
+  er = errno;
+
+  if (gnut_lib_debug >= level) {
+    pthread_mutex_lock(&_g_debug_mutex);
+    GDFLUSH_O;
+    printf("%s: %s\n", s, strerror(er));
+    GDFLUSH_E;
+    pthread_mutex_unlock(&_g_debug_mutex);
+  }
+#endif
+}
+
+#endif /* GDEBUG_ENABLE */
+
 /* ----------------------------- unclassified ----------------------------- */
 
 char *gnut_strdelimit(char *string, char *delim, char new_delim)
 {
   char *c;
-  
+
   for (c=string;*c;c++) {
     if (strchr(delim,*c)) *c=new_delim;
   }
-  
+
   return string;
 }
 
@@ -1044,7 +1328,7 @@ void gnut_strstrip(char *string)
 {
   int i;
 
-  for (i=(strlen(string)-1);i>0 && isspace(string[i]);i--) {
+  for (i=(strlen(string)-1);i>0 && G_ISSPACE(string[i]);i--) {
     string[i]=0;
   }
 }
@@ -1056,42 +1340,42 @@ char *munge_time(int secs)
   char buf[10];
   int days, hours, minutes, seconds;
   int fmt;
-  
+
   s = ymaloc(40*sizeof(char), 323);
   days = hours = minutes = seconds = 0;
   strcpy(s,"");
-  
+
   if (secs==0) {
     strcat(s,"0s");
     return s;
   }
-  
+
   if (secs<0) {
     /* handle negatives correctly */
     strcat(s,"-");
     secs=-secs;
   }
-  
+
   seconds=secs;
-  
+
   if (seconds > 60) {
     /* how many minutes? */
     minutes=seconds / 60;
     seconds=seconds % 60;
   }
-  
+
   if (minutes >= 60) {
     /* hours? */
     hours=minutes / 60;
     minutes = minutes %60;
   }
-  
+
   if (hours >= 24) {
     /* and the days */
     days=hours/24;
     hours=hours % 24;
   }
-  
+
   /* build the string */
   fmt = 0;
   if (days) {
@@ -1101,7 +1385,7 @@ char *munge_time(int secs)
   }
   if (hours || fmt) {
     sprintf(buf, "%dh", hours);
-    strcat(s, buf); 
+    strcat(s, buf);
     fmt = 1;
   }
   if (minutes || fmt) {
@@ -1114,7 +1398,7 @@ char *munge_time(int secs)
     strcat(s, buf);
     fmt = 1;
   }
-  
+
   return s;
 }
 
@@ -1124,7 +1408,7 @@ char *format_si(float64 x, char *s)
    * letters of the Latin alphabet, starting from the last and moving
    * backwards. Since Latin has no J or W, the prefixes after Z and Y will
    * be X, V, and U; after that I don't know what to predict because the
-   * letter T is already used for tera. 
+   * letter T is already used for tera.
    *
    * Refer to: "The Peta-Principle" by Jim Binder, NIST publictaion
    * SP811 "Guide for the Use of the International System of Units (SI)",
@@ -1169,13 +1453,11 @@ int trycmd(char *cmd, char *pat)
   char inbuf[100];
   int rv;
 
-  gd_s(2, "trycmd '");
-  gd_s(2, cmd);
-  gd_s(2, "'\n");
+  GD_S(2, "trycmd '"); GD_S(2, cmd); GD_S(2, "'\n");
 
   pipe = popen(cmd, "r");
   if (pipe == 0) {
-    gd_s(2, "trycmd openpipe failed\n");
+    GD_S(2, "trycmd openpipe failed\n");
     return(0);
   }
 
@@ -1184,9 +1466,7 @@ int trycmd(char *cmd, char *pat)
    * that the command exists. */
   rv = 0;
   while (fgets(inbuf, 100, pipe)) {
-    gd_s(3, "trycmd read '");
-    gd_s(3, inbuf);
-    gd_s(3, "'\n");
+    GD_S(3, "trycmd read '"); GD_S(3, inbuf); GD_S(3, "'\n");
     if(strstr(inbuf, pat)) {
       rv = 1;
     }
@@ -1211,7 +1491,7 @@ int keyword_match(char *q1, char *s1, int and, int ignorecase)
   int going;
   char *loc;
 
-  gd_s(5, "ismatch_tok entering\n");
+  GD_S(5, "ismatch_tok entering\n");
 
   q2 = s2 = 0;
 
@@ -1246,11 +1526,8 @@ int keyword_match(char *q1, char *s1, int and, int ignorecase)
   }
   going = 1;
   while (a && going) {
-    gd_s(5, "ismatch_tok checking for \"");
-    gd_s(5, a);
-    gd_s(5, "\" in \"");
-    gd_s(5, str);
-    gd_s(5, "\" \n");
+    GD_S(5, "ismatch_tok checking for \""); GD_S(5, a); GD_S(5, "\" in \"");
+    GD_S(5, str); GD_S(5, "\" \n");
     loc = strstr(str, a);
     if (loc == 0) {
       /* Not found */
@@ -1287,9 +1564,9 @@ int keyword_match(char *q1, char *s1, int and, int ignorecase)
  km_exit1: ;
 
   if (result == 0) {
-    gd_s(5, "ismatch_tok returning failure\n");
+    GD_S(5, "ismatch_tok returning failure\n");
   } else {
-    gd_s(5, "ismatch_tok returning success\n");
+    GD_S(5, "ismatch_tok returning success\n");
   }
 
   /* Free up any copies we made */
