@@ -5,13 +5,6 @@
 #ifndef GNUT_LIST_H
 #define GNUT_LIST_H
 
-typedef struct _Gnut_List Gnut_List;
-
-struct _Gnut_List {
-  void * data;
-  Gnut_List * next;
-};
-
 #ifdef __cplusplus
 extern "C" {
 #endif
