@@ -14,16 +14,13 @@
 # include <unistd.h>
 #endif
 
+#include "gnut.h" /* 0.4.28.c01 */
 #include "monitor.h"
 #include "queue.h"
 
 Gnut_Queue * monitor_queue;
 
-#ifndef PTHREADS_DRAFT4
- pthread_mutex_t monitor_mutex = PTHREAD_MUTEX_INITIALIZER;
-#else
- pthread_mutex_t monitor_mutex;
-#endif
+/* 0.4.28.c01 Moved mutex to gnut.c */
 
 void fre_ms(monitored_search **x, int bugnum)
 {
@@ -45,7 +42,7 @@ int monitor_search_add(char *search, int matched)
   monitored_search *ms;
   uchar * s;
 
-  gd_s(3, "monitor_search_add entering\n");
+  GD_S(3, "monitor_search_add entering\n");
 
   ms = (monitored_search *)ymaloc(sizeof(monitored_search), 305);
   ms->search = ystdup(search, 454);
@@ -61,7 +58,7 @@ int monitor_search_add(char *search, int matched)
   gnut_queue_insert(monitor_queue, ms, 505);
   pthread_mutex_unlock(&monitor_mutex);
 
-  gd_s(3, "monitor_search_add leaving\n");
+  GD_S(3, "monitor_search_add leaving\n");
   return 0;
 }
 
