@@ -2,41 +2,41 @@
 
 /* This file is distributed under the GPL, see file COPYING for details */
 
+/* level-1 system headers */
 #include <stdio.h>
 #include <stdlib.h>
-#include <sys/types.h>
-
-#ifndef WIN32
-# include <sys/time.h>
-# include <sys/socket.h>
-# include <netinet/in.h>
-# include <netinet/tcp.h>
-# include <arpa/inet.h>
-# include <sys/ioctl.h>
-# include <unistd.h>
-# include <net/if.h>
-# include <netdb.h>
-#endif
-
+#include "sh_sys_types.h"
+#include <sys/time.h>
+#include <sys/socket.h>
+#include <netinet/in.h>
+#include <netinet/tcp.h>
+#include <arpa/inet.h>
+#include <sys/ioctl.h>
+#include <unistd.h>
+#include <net/if.h>
+#include <netdb.h>
 #include <errno.h>
 #include <string.h>
 #include <ctype.h>
 #include <fcntl.h>
 #include <time.h>
 #include <pthread.h>
-#include <regex.h>
-/* 0.4.27.c08 removed #include assert.h */
 
+/* level-2 system headers */
+#include "sv_regex.h"
+
+/* local system headers */
+#include "blacklist.h"
 #include "conf.h"
 #include "connection.h"
-#include "lib.h"
-#include "threads.h"
-#include "if.h"
-#include "net.h"
-#include "ui.h"
+#include "gnut.h" /* 0.4.28.c01 */
 #include "host.h"
+#include "if.h"
+#include "lib.h"
+#include "net.h"
 #include "protocol.h"
-#include "blacklist.h"
+#include "threads.h"
+#include "ui.h"
 
 uchar local_ip[4];
 
@@ -59,18 +59,18 @@ int gnut_net_host()
   struct hostent *he;
   int ret;
 
-  gd_s(2,"gnut_net_host entering\n");
+  GD_S(2,"gnut_net_host entering\n");
 
   ret = gethostname(host, sizeof(host));
 
   if (ret < 0) {
-    perror("gnut_net_host, gethostname");
+    GD_PERROR(0, "gnut_net_host, gethostname");
     return ret;
   }
 
   he = gethostbyname(host);
   if (he==0) {
-    perror("gnut_net_host, gethostbyname");
+    GD_PERROR(0, "gnut_net_host, gethostbyname");
     return ret;
   }
 
@@ -86,28 +86,28 @@ int net_init(char *iface)
 {
   struct sockaddr_in *sinptr = 0;
 
-  gd_s(2, "net_init entering\n");
+  GD_S(2, "net_init entering\n");
 
   if (iface) {
     if (inet_aton(iface,(struct in_addr *) local_ip)) {
-      gd_s(1, "local was set by command to: "); gd_i(1, local_ip[0]);
-      gd_s(1, "."); gd_i(1, local_ip[1]); gd_s(1, "."); gd_i(1, local_ip[2]);
-      gd_s(1, "."); gd_i(1, local_ip[3]); gd_s(1, "\n");
+      GD_S(1, "local was set by command to: "); GD_I(1, local_ip[0]);
+      GD_S(1, "."); GD_I(1, local_ip[1]); GD_S(1, "."); GD_I(1, local_ip[2]);
+      GD_S(1, "."); GD_I(1, local_ip[3]); GD_S(1, "\n");
       return 0;
     }
   }
 
   if ((sinptr = get_if_addr(iface)) == 0) {
-    gd_s(1, "Can't get local IP address through interface, trying"
-            " host name...\n");
+    GD_S(1, "Can't get local IP address through interface, trying"
+	 " host name...\n");
     gnut_net_host();  
   } else {
     memcpy(local_ip,&(sinptr->sin_addr),sizeof(local_ip));
   }
 
-  gd_s(1,"local ip is: "); gd_i(1, local_ip[0]); gd_s(1, ".");
-  gd_i(1, local_ip[1]); gd_s(1, "."); gd_i(1, local_ip[2]); gd_s(1, ".");
-  gd_i(1, local_ip[3]); gd_s(1, "\n");
+  GD_S(1,"local ip is: "); GD_I(1, local_ip[0]); GD_S(1, ".");
+  GD_I(1, local_ip[1]); GD_S(1, "."); GD_I(1, local_ip[2]); GD_S(1, ".");
+  GD_I(1, local_ip[3]); GD_S(1, "\n");
 
   printf("VPN class %i\n", host_class(local_ip));
 
@@ -141,7 +141,7 @@ int timed_read(int sock, char *buf, int len, int secs)
 
     ret = select(sock+1, &fsr, 0, 0, &tv);
     if (ret<0) {
-      perror("select");
+      GD_PERROR(0, "timed_read, select");
     }
 
     if (ret==0) {
@@ -227,33 +227,18 @@ int timed_connect(int sockfd, struct sockaddr *sa, int addrlen, int secs,
   int val, len;
   int countdown, result;
 
-  gd_s(1, "timed.conn entering sock="); gd_i(1, sockfd); gd_s(1, " secs=");
-  gd_i(1, secs); gd_s(1, "\n");
+  GD_S(1, "timed.conn entering sock="); GD_I(1, sockfd); GD_S(1, " secs=");
+  GD_I(1, secs); GD_S(1, "\n");
 
   ret = fcntl(sockfd, F_SETFL, O_NONBLOCK);
-  gd_s(5, "timed.conn fcntl returned "); gd_i(5, ret); gd_s(5, "\n");
+  GD_S(5, "timed.conn fcntl returned "); GD_I(5, ret); GD_S(5, "\n");
 
   if (ret<0) {
     return ret;
   }
 
   ret = connect(sockfd, sa, addrlen);
-  gd_s(5, "timed.conn connect returned "); gd_i(5, ret); gd_s(5, "\n");
-
-#if 0
-  if (ret==0) {
-    /* wooah!  immediate connection */
-    gd_s(0, "timed.conn immediate connection!\n");
-    return -2;
-  }
-#endif
-
-#if 0
-  if (errno != EINPROGRESS) {
-    perror("timed.conn, connect");
-    return ret;
-  }
-#endif  
+  GD_S(5, "timed.conn connect returned "); GD_I(5, ret); GD_S(5, "\n");
 
   FD_ZERO(&fsr);
   FD_SET(sockfd, &fsr);
@@ -274,7 +259,7 @@ int timed_connect(int sockfd, struct sockaddr *sa, int addrlen, int secs,
     tv.tv_usec = 0;
 
     ret = select(sockfd+1, 0, &fsr, 0, &tv);
-    gd_s(5,"timed.conn select returned "); gd_i(5, ret); gd_s(5, "\n");
+    GD_S(5,"timed.conn select returned "); GD_I(5, ret); GD_S(5, "\n");
 
     if (ret == 0) {
       /* timeout, keep on looping */
@@ -285,7 +270,7 @@ int timed_connect(int sockfd, struct sockaddr *sa, int addrlen, int secs,
 
     /* Allow user to stop the connection */
     if (gt) {
-      if (gt->state != STATE_CONNECTING) {
+      if (gt->gt_state != STATE_CONNECTING) {
 	if (gc_debug_opts & 8) {
 	  printf("timed.conn detect cancel by user\n");
 	  if(gt) {
@@ -300,7 +285,7 @@ int timed_connect(int sockfd, struct sockaddr *sa, int addrlen, int secs,
 
   if (result == 0) {
     /* timeout */
-    gd_s(1, "timed.conn timeout\n");
+    GD_S(1, "timed.conn timeout\n");
     fcntl(sockfd, F_SETFL, 0);
     return -2;
   } else if (result == 2) {
@@ -310,23 +295,23 @@ int timed_connect(int sockfd, struct sockaddr *sa, int addrlen, int secs,
 
   len = 4;
   ret = getsockopt(sockfd, SOL_SOCKET, SO_ERROR, &val, &len);
-  gd_s(5,"timed.conn getsockopt returned "); gd_i(5, ret); gd_s(5," val=");
-  gd_i(5, val); gd_s(5, "\n");
+  GD_S(5, "timed.conn getsockopt returned "); GD_I(5, ret); GD_S(5," val=");
+  GD_I(5, val); GD_S(5, "\n");
 
   if (ret < 0) {
-    gd_s(1,"timed.conn getsockopt returned: "); gd_i(1, ret); gd_s(1, "\n");
+    GD_S(1, "timed.conn getsockopt returned: "); GD_I(1, ret); GD_S(1, "\n");
     return ret;
   }
 
   if (val != 0) {
-    gd_s(3,"timed.conn returning failure!\n");
+    GD_S(3,"timed.conn returning failure!\n");
     return -1;
   }
 
   ret = fcntl(sockfd, F_SETFL, 0);
 
-  gd_s(1,"timed.conn fcntl: "); gd_i(1, ret); gd_s(1, "\n");
-  gd_s(3,"timed.conn returning success val="); gd_i(3, val); gd_s(3, "\n");
+  GD_S(1,"timed.conn fcntl: "); GD_I(1, ret); GD_S(1, "\n");
+  GD_S(3,"timed.conn returning success val="); GD_I(3, val); GD_S(3, "\n");
   return 0;
 }
 
@@ -342,33 +327,34 @@ int create_listen_socket(int *port)
   int ret,i;
   int val;
 
-  gd_s(3, "create_listen_socket entering\n");
+  GD_S(3, "create_listen_socket entering\n");
 
   sock = socket(AF_INET, SOCK_STREAM, 6);
   if (sock<0) {
-    perror("create_listen_socket, socket");
+    GD_PERROR(0, "create_listen_socket, socket");
     return sock;
   }
 
   val = 1;
   ret = setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, &val, sizeof(val));
 
-  /* Turn off the Nagle "tinygram prevention" algorithm. (CURRENTLY
-   * INSIDE IF(0))
-   * 
-   * (Background: The Nagle algorithm is an option that applies to any
-   * TCP connection. It causes data to be grouped into large blocks
-   * before being sent, unless the other end is known to be waiting
-   * for more data. The other end is waiting for data if we have
-   * received ACKs for al as-yet-sent packets.)
-   *
-   * By turning off Nagle, we enable the sending of small packets even
-   * when the other end has not yet acknowledged all data we have sent
-   * it.
-   *
-   * Someone suggested this change, and I subsequently discovered that
-   * it does not actually benefit anything except the startup time for
-   * GnutellaNet connections -- so it is here but disabled. */
+  /* This disabled code turns off the Nagle "tinygram prevention"
+     algorithm.
+
+     (Background: The Nagle algorithm is an option that applies to any
+     TCP connection. It causes data to be grouped into large blocks
+     before being sent, unless the other end is known to be waiting
+     for more data. The other end is waiting for data if we have
+     received ACKs for al as-yet-sent packets.)
+    
+     By turning off Nagle, we enable the sending of small packets even
+     when the other end has not yet acknowledged all data we have sent
+     it.
+    
+     Someone suggested this change, but due to the nature of the Gnutella
+     protocol it does not actually confer any benefit. "Gnutella packets"
+     are not packets at all, but actually just blocks of bytes within a TCP
+     stream. Therefore, the code is here but disabled. */
 #if 0
   int flag;
 
@@ -379,7 +365,7 @@ int create_listen_socket(int *port)
 		    (char *) &flag,    /* the cast is historical cruft */
 		    sizeof (int));      /* length of option value */
   if (ret < 0) {
-    perror ("setsockopt() - TCP_NODELAY");
+    GD_PERROR("setsockopt() - TCP_NODELAY");
     return (ret);
   }
 #endif
@@ -397,24 +383,24 @@ int create_listen_socket(int *port)
     }
   }
   if (ret < 0) {
-    perror("create_listen_socket, bind");
+    GD_PERROR(0, "create_listen_socket, bind");
     return ret;
   }
 
   ret = listen(sock,5);
   if (ret<0) {
-    perror("create_listen_socket, listen");
+    GD_PERROR(0, "create_listen_socket, listen");
     return ret;
   }
 
-  gd_s(1, "create_listen_socket bound to port "); gd_i(1, *port);
-  gd_s(1, "\n");
+  GD_S(1, "create_listen_socket bound to port "); GD_I(1, *port);
+  GD_S(1, "\n");
 
   /* printf("Bound to port: %i\n",*port); */
 
   ret = fcntl(sock, F_SETFL, O_NONBLOCK);
   if (ret<0) {
-    perror("create_listen_socket, fcntl");
+    GD_PERROR(0, "create_listen_socket, fcntl");
     return ret;
   }
 
@@ -433,7 +419,7 @@ int read_line(int sock, char *buf, int len)
   char *ptr;
 
   ptr = buf;
-  gd_s(3, "read_line entering\n");
+  GD_S(3, "read_line entering\n");
   fcntl(sock, F_SETFL, 0);
 
   buf[0] = 0;
@@ -462,23 +448,17 @@ int read_line(int sock, char *buf, int len)
         goto try_again;
       }
 
-      if (gnut_lib_debug>=1) {
-	perror("read_line, read");
-      }
+      GD_PERROR(1, "read_line, read");
       return -1;
     }
   }
   buf[i]=0;
 
-  gd_s(3, "read_line returning success\n");
+  GD_S(3, "read_line returning success\n");
   return i;
 }    
 
-#ifndef PTHREADS_DRAFT4
-pthread_mutex_t make_conn_mutex=PTHREAD_MUTEX_INITIALIZER;
-#else
-pthread_mutex_t make_conn_mutex;
-#endif
+/* 0.4.28.c01 Moved mutex to gnut.c */
 
 struct hostent *gnut_hostent_copy(struct hostent *he)
 {
@@ -511,29 +491,30 @@ void gnut_hostent_delete(struct hostent *he)
 {
   int i, len;
 
-  gd_s(1,"gnut_hostent_delete entering\n");
+  GD_S(1,"gnut_hostent_delete entering\n");
 
-  for (len=0; he->h_addr_list[len]; len++) ;
+  for (len=0; he->h_addr_list[len]; len++) {
+  }
 
-  gd_s(1, "gnut_hostent_delete len="); gd_i(1, len); gd_s(1, "\n");
+  GD_S(1, "gnut_hostent_delete len="); GD_I(1, len); GD_S(1, "\n");
   for (i=0; i<len; i++) {
-    gd_s(1, "gnut_hostent_delete deleting i="); gd_i(1, i); gd_s(1, "\n");
+    GD_S(1, "gnut_hostent_delete deleting i="); GD_I(1, i); GD_S(1, "\n");
     if (he->h_addr_list[i]) {
       fre_str(&(he->h_addr_list[i]), 257);
     }
   }
 
-  gd_s(1, "gnut_hostent_delete fr""eeing h_name\n");
+  GD_S(1, "gnut_hostent_delete fr""eeing h_name\n");
   if (he->h_name) {
     fre_str(&(he->h_name), 258);
   }
 
-  gd_s(1, "gnut_hostent_delete fr""eeing h_addr_list\n");
+  GD_S(1, "gnut_hostent_delete fr""eeing h_addr_list\n");
   fre_strl(&(he->h_addr_list), 259);
 
-  gd_s(1, "gnut_hostent_delete fr""eeing he\n");
+  GD_S(1, "gnut_hostent_delete fr""eeing he\n");
   fre_he(&he, 260);
-  gd_s(1, "gnut_hostent_delete returning\n");
+  GD_S(1, "gnut_hostent_delete returning\n");
 }
   
 
@@ -553,13 +534,13 @@ int make_connection(uchar *hname, uint port, uchar ip[4], int for_transfer,
   int timeout;
   char buf_ip[100];
 
-  gd_s(1,"make_conn entering\n");
+  GD_S(1,"make_conn entering\n");
 
   pthread_mutex_lock(&make_conn_mutex);
 
   he = gnut_hostent_copy(gethostbyname(hname));
   if (he == 0) {
-    gd_s(1, "make_conn No such host: "); gd_s(1, hname); gd_s(1, "\n");
+    GD_S(1, "make_conn No such host: "); GD_S(1, hname); GD_S(1, "\n");
     pthread_mutex_unlock(&make_conn_mutex);
     return -1;
   }
@@ -576,10 +557,10 @@ int make_connection(uchar *hname, uint port, uchar ip[4], int for_transfer,
 	    (uchar) he->h_addr_list[i][0], (uchar) he->h_addr_list[i][1],
 	    (uchar) he->h_addr_list[i][2], (uchar) he->h_addr_list[i][3],
 	    port);
-    gd_s(1, "make_conn trying host "); gd_s(1, buf_ip); gd_s(1, "\n");
+    GD_S(1, "make_conn trying host "); GD_S(1, buf_ip); GD_S(1, "\n");
     sock = socket(AF_INET, SOCK_STREAM, 6);
     if (sock<0) {
-      perror("make_conn, socket");
+      GD_PERROR(0, "make_conn, socket");
       gnut_hostent_delete(he);
       return sock;
     }
@@ -596,11 +577,11 @@ int make_connection(uchar *hname, uint port, uchar ip[4], int for_transfer,
     }
     ret = timed_connect(sock, (struct sockaddr *) &sin,
 			sizeof(struct sockaddr_in), timeout, gt);
-    gd_s(5, "make_conn timed_conn returned: "); gd_i(5, ret); gd_s(5, "\n");
+    GD_S(5, "make_conn timed_conn returned: "); GD_I(5, ret); GD_S(5, "\n");
     if (ret<0) {
-      if (for_transfer && gt) { /* 0.4.27.c03 */
-	int i1; /* 0.4.27.c03 This is just to eliminate the duplication of
-                   calls to printf for the three different versions */
+      if (for_transfer && gt) {
+	int i1; /* This is just to eliminate the duplication of calls
+ 	           to printf for the three different versions */
 	i1 = 0;
 	if (gt->gt_dest_cache) {
 	  if (gc_verbose & 4) {
@@ -623,7 +604,7 @@ int make_connection(uchar *hname, uint port, uchar ip[4], int for_transfer,
 	  printf("  %s\n", gt->fname);
 	}
       }
-      gd_s(1,"make_conn host bad, closing\n");
+      GD_S(1,"make_conn host bad, closing\n");
       close_s(sock);
     } else {
       /* Successful */
@@ -631,7 +612,7 @@ int make_connection(uchar *hname, uint port, uchar ip[4], int for_transfer,
     }
   }
 
-  if (gt && (gt->state != STATE_CONNECTING)) {
+  if (gt && (gt->gt_state != STATE_CONNECTING)) {
     if(gt->type == TYPE_RECV_PUSH) {
       if (gc_verbose & 2) {
 	printf("Detect PUSH connection stopped while opening\n");
@@ -651,19 +632,19 @@ int make_connection(uchar *hname, uint port, uchar ip[4], int for_transfer,
   }
 
   if ((ret < 0) || (he->h_addr_list[i] == 0)) {
-    gd_s(1,"make_conn returning failure\n");
+    GD_S(1,"make_conn returning failure\n");
     gnut_hostent_delete(he);
     return -2;
   }
-  gd_s(5, "make_conn about to copy ip from slot "); gd_i(5, i); gd_s(5, "\n");
+  GD_S(5, "make_conn about to copy ip from slot "); GD_I(5, i); GD_S(5, "\n");
 
   memcpy(ip, he->h_addr_list[i], 4);
 
-  gd_s(4, "make_conn trying to unlock mutex\n");
+  GD_S(4, "make_conn trying to unlock mutex\n");
 
   gnut_hostent_delete(he);
 
-  gd_s(1, "make_conn returning success\n");
+  GD_S(1, "make_conn returning success\n");
 
   return sock;
 }
@@ -671,43 +652,38 @@ int make_connection(uchar *hname, uint port, uchar ip[4], int for_transfer,
 /* int send_packet (int sock, gnutella_packet *gpa)
  *
  * sends the packet described by gpa over socket sock
- * return 0 on success or <0 for error */
+ * returns number of bytes written or negative error code */
 int send_packet(int sock, gnutella_packet *gpa)
 {
   int ret;
-  int t;
+  uint32 dlen;
 
-  gd_s(3,"send_packet entering\n");
+  GD_S(3,"send_packet entering\n");
 
   ret = write(sock, (char *) &gpa->gh, sizeof(gnutella_hdr));
   if (ret<0) {
-    if (gnut_lib_debug>3) {
-      perror("send_packet, write header");
-    }
+    GD_PERROR(4, "send_packet, write header");
     return ret;
   }
 
-  memcpy(&t, gpa->gh.dlen, 4);
-  t = GUINT32_FROM_LE(t);
+  dlen = GET_LEU32(gpa->gh + PROT_HDR_DLEN, 23);
 
-  if (t>0) {
-    ret = write(sock, gpa->data, t);
+  if (dlen) {
+    ret = write(sock, gpa->data, dlen);
     if (ret<0) {
-      if (gnut_lib_debug>3) {
-        perror("send_packet, write data");
-      }
+      GD_PERROR(4, "send_packet, write data");
       return ret;
     }
   }
-  gd_s(3, "send_packet returning success\n");
+  GD_S(3, "send_packet returning success\n");
 
-  return 23+t;
+  return 23 + dlen;
 }
 
 /* int read_packet(int sock, gnutella_packet *gpa)
  *
  * reads a packet from the socket sock into the packet structure of gpa.
- * returns 0 on success or <0 on error */
+ * returns number of bytes read or negative error code */
 int read_packet(int sock, gnutella_packet *gpa)
 {
   int ret;
@@ -716,21 +692,19 @@ int read_packet(int sock, gnutella_packet *gpa)
   uint dlen;
   int i;
 
-  gd_s(3, "read_pkt entering\n");
+  GD_S(3, "read_pkt entering\n");
 
   ptr = (char *) &gpa->gh;
   for (left=sizeof(gnutella_hdr); left>0; ) {
     ret = timed_read(sock, ptr, left, 10);
-    gd_s(6, "read_pkt timed_read ret "); gd_i(6, ret); gd_s(6, "\n");
+    GD_S(6, "read_pkt timed_read ret "); GD_I(6, ret); GD_S(6, "\n");
     if (ret == -2) {
       return 0;
     } else if (ret == 0) {
       return -2;
     } else if (ret<0) {
       if (errno != EINTR) {
-        if (gnut_lib_debug>3) {
-	  perror("read_packet, header");
-	}
+        GD_PERROR(4, "read_packet, header");
         return ret;
       } else {
 	ret = 0;
@@ -740,24 +714,21 @@ int read_packet(int sock, gnutella_packet *gpa)
     ptr += ret;
     left -= ret;
   }
-  /* 0.4.27.c08 Removed assert here, the conditions of the for loop
-     prevented it from ever happening anyway */
 
-  memcpy(&dlen, gpa->gh.dlen, 4);
-  dlen = GUINT32_FROM_LE(dlen);
+  dlen = GET_LEU32(gpa->gh + PROT_HDR_DLEN, 24);
 
-  while (dlen>32767) {
+  while (dlen > 32767) {
     /* We're out of sync!  I'm going to do large reads until
      * one returns 23, which is probably a ping packet.... */
-    gd_s(2, "read_pkt out of sync! func="); gd_i(2, gpa->gh.func);
-    gd_s(2, " dlen="); gd_i(2, dlen); gd_s(2, "\n");
+    GD_S(2, "read_pkt out of sync! func="); GD_I(2, gpa->gh[PROT_HDR_FUNC]);
+    GD_S(2, " dlen="); GD_I(2, dlen); GD_S(2, "\n");
     ptr = (char *) ymaloc(100, 320);
 
     ret = 1;
     i = 0;
-    while (ret>0 && ret!=23 && ret!=49) {
+    while ((ret>0) && (ret!=23) && (ret!=49)) {
       ret = timed_read(sock, ptr, 60, 2);
-      gd_s(6, "read_pkt timed_read ret "); gd_i(6, ret); gd_s(6, "\n");
+      GD_S(6, "read_pkt timed_read ret "); GD_I(6, ret); GD_S(6, "\n");
       if ((ret == -2) || (++i>5)) {
         fre_str(&ptr, 261);
         return 0;
@@ -770,8 +741,10 @@ int read_packet(int sock, gnutella_packet *gpa)
     }
 
     memcpy(&gpa->gh, ptr, 23);
-    memcpy(&dlen, gpa->gh.dlen, 4);
-    dlen = GUINT32_FROM_LE(dlen);
+
+    /* Get the dlen of this new packet */
+    dlen = GET_LEU32(gpa->gh + PROT_HDR_DLEN, 25);
+
     fre_str(&ptr, 263);
   }
 
@@ -780,15 +753,13 @@ int read_packet(int sock, gnutella_packet *gpa)
     ptr = gpa->data;
     for (left=dlen; left>0; ) {
       ret = timed_read(sock, ptr, left, 10);
-      gd_s(6, "read_pkt timed_read ret "); gd_i(6, ret); gd_s(6, "\n");
+      GD_S(6, "read_pkt timed_read ret "); GD_I(6, ret); GD_S(6, "\n");
       if (ret==-2) {
 	return 0;
       } else if (ret==0) {
 	return -2;
       } else if (ret<0) {
-        if (gnut_lib_debug>3) {
-	  perror("read_packet, data");
-	}
+        GD_PERROR(4,"read_packet, data");
         return ret;
       }
 
@@ -798,8 +769,8 @@ int read_packet(int sock, gnutella_packet *gpa)
   }
 
   if (dlen>3000) {
-    gd_s(2,"read_pkt OVERSIZE packet! size: "); gd_i(2, dlen); gd_s(2, "\n");
+    GD_S(2,"read_pkt OVERSIZE packet! size: "); GD_I(2, dlen); GD_S(2, "\n");
   }
-  gd_s(3,"read_pkt returning success\n");
+  GD_S(3,"read_pkt returning success\n");
   return 23+dlen;
 }
