@@ -8,14 +8,6 @@
 #include "lib.h"
 #include "list.h"
 
-typedef struct _Gnut_Queue Gnut_Queue;
-
-struct _Gnut_Queue {
-  Gnut_List *head;
-  Gnut_List *tail;
-  uint32 size;
-};
-
 #ifdef __cplusplus
 extern "C" {
 #endif
