@@ -25,7 +25,7 @@ extern "C" {
 
   void fre_re(route_entry **x, int bugnum);
   int route_guid_add(uchar guid[16], gcs *gc);
-  gcs * route_guid_find(uchar guid[16], route_entry **the_re);
+  gcs * route_guid_find(uchar * guid, route_entry **the_re);
   int route_guid_clear(gcs *gc);
   int route_init();
 
