@@ -8,19 +8,19 @@
 #include <stdlib.h>
 #include <sys/param.h>
 #include <sys/stat.h>
-#include <sys/types.h>
+
+/* 0.4.28.c17 */
+#include "sh_sys_types.h"
+
 #include <signal.h>
-
-#ifndef WIN32
 #include <unistd.h>
-#endif
-
 #include <errno.h>
 #include <string.h>
 #include <pthread.h>
 #include <ctype.h>
 
 #include "conf.h"
+#include "gnut.h" /* 0.4.28.c01 */
 #include "hash.h"
 #include "lib.h"
 #include "list.h"
@@ -29,7 +29,7 @@
 
 
 /* here we declare share's global variables... */
-uint32 g_share_num = 0; /* 0.4.27.c11 */
+uint32 g_share_num = 0;
 float64 g_share_size = 0;
 Gnut_List *share_root = 0;
 
@@ -37,17 +37,11 @@ int sh_dbg_1;
 int sh_qreplies_active;
 int sh_qreplies_hi;
 
-#ifndef PTHREADS_DRAFT4
-pthread_mutex_t share_mutex=PTHREAD_MUTEX_INITIALIZER;
-#else
-pthread_mutex_t share_mutex;
-#endif
+/* 0.4.28.c01 Moved mutex to gnut.c */
 
-/* 0.4.27.c17 */
-#if 0
-Gnut_Hash *share_cache_hash = 0;
-Gnut_Queue *share_cache_queue = 0;
-#endif
+/* 0.4.28.c05 Removed the old Josh "share cache", an attempt at cacheing
+   the results of share_search which was always disabled and wouldn't have
+   worked too well either. The code I have removed was all inside #if 0 */
 
 int have_find, have_perl;
 
@@ -83,7 +77,7 @@ int sh_tokenize(char *str, char **tokens, int maxtokens)
   i = 0;
   while((str[i]) && (nt < maxtokens)) {
     ia_last = ia_this;
-    ia_this = (isalnum(str[i]));
+    ia_this = (G_ISALNUM(str[i]));
     if (ia_this && (ia_last == 0)) {
       /* Beginning of a word */
       tokens[nt++] = str+i;
@@ -102,8 +96,7 @@ int sh_tokenize(char *str, char **tokens, int maxtokens)
  * perform the query against is s1.
  *  match.tok_2 always does an AND search, and does not ignore case (because
  * both arguments are already lowercased before match.tok_2 is called)
- *  must_exact (which is for 0.4.27.c04)
-               should be set if you want it to match only if *all* of the
+ *  must_exact should be set if you want it to match only if *all* of the
  * letters and numbers in s1 match a letter or number in the tokens.
  *  The return value is nonzero if there is a match.
  * WARNING: s1 must be 7-bit ASCII, and if not its high bits will be cleared
@@ -115,7 +108,7 @@ int match_tok_2(char **tokens, int nt, char *s1, int must_exact)
   int result;
   uint8 *p;
   int ftnum;
-  int isexact; /* 0.4.27.c04 */
+  int isexact;
 
   if (nt <= 0) {
     return 0;
@@ -162,14 +155,14 @@ int match_tok_2(char **tokens, int nt, char *s1, int must_exact)
 
   /* If must_exact, scan to see if there are any as-yet-untagged alphanumeric
      characters */
-  isexact = 1; /* 0.4.27.c04 */
-  if (result && must_exact) { /* 0.4.27.c04 */
+  isexact = 1;
+  if (result && must_exact) {
     uint8 c;
 
     p = s1;
     for(j=0; p[j] && isexact; j++) {
       c = p[j];
-      if (isalnum(c)) {
+      if (G_ISALNUM(c)) {
 	isexact = 0;
       }
     }
@@ -190,7 +183,7 @@ int match_tok_2(char **tokens, int nt, char *s1, int must_exact)
     }
   }
 
-  return (result && isexact); /* 0.4.27.c04 */
+  return (result && isexact);
 }
 
 /* Gnut_List *share.search(char *query, int max)
@@ -214,7 +207,7 @@ Gnut_List *share_search(char *orig_query, int max, int *n_reslts)
   nlist = 0;
   *n_reslts = 0;
 
-  gd_s(3, "share.search entering\n");
+  GD_S(3, "share.search entering\n");
 
   tok_query = ystdup(orig_query, 438);
   dqi(0x0077);
@@ -249,8 +242,8 @@ Gnut_List *share_search(char *orig_query, int max, int *n_reslts)
   for (ltmp=share_root; ltmp; ltmp=ltmp->next) {
     sh_dbg_1++;
 
-    gd_s(4, "share.search ltmp="); gd_p(4, ltmp); gd_s(4, " orig_query=");
-    gd_p(4, orig_query); gd_s(4, "\n");
+    GD_S(4, "share.search ltmp="); GD_P(4, ltmp); GD_S(4, " orig_query=");
+    GD_P(4, orig_query); GD_S(4, "\n");
     /* see if this share matches... */
     si = ltmp->data;
 
@@ -267,7 +260,7 @@ Gnut_List *share_search(char *orig_query, int max, int *n_reslts)
     }
 #endif
 
-    if (match_tok_2(tokens, nt, si->lc_path, 0)) /* 0.4.27.c04 */
+    if (match_tok_2(tokens, nt, si->lc_path, 0))
       {
 	/* we need to make a copy of this node, and
 	 * insert into the result list... */
@@ -295,7 +288,7 @@ Gnut_List *share_search(char *orig_query, int max, int *n_reslts)
 
   *n_reslts = count;
   
-  gd_s(3, "share.search returning\n");
+  GD_S(3, "share.search returning\n");
   return nlist;  
 } 
 
@@ -311,15 +304,15 @@ Gnut_List *share_get_root()
  *
  * returns total number of shared files, and their total size
  * in kbytes (that 'k' is the SI k = 1000, not 1024!) */
-void share_totals(uint32 *sh_num, float64 *sh_size) /* 0.4.27.c11 */
+void share_totals(uint32 *sh_num, float64 *sh_size)
 {
-  gd_s(3,"share.totals entering\n");
+  GD_S(3,"share.totals entering\n");
   *sh_num = g_share_num;
   *sh_size = g_share_size;
 
-  gd_s(3, "share.totals returning share_num="); gd_u(3, g_share_num);
-  gd_s(3, " share_size="); gd_f64(3, g_share_size); gd_s(3, " kbytes  *num=");
-  gd_u(3, *sh_num); gd_s(3, " *size="); gd_f64(3, *sh_size); gd_s(3, "\n");
+  GD_S(3, "share.totals returning share_num="); GD_U(3, g_share_num);
+  GD_S(3, " share_size="); GD_F64(3, g_share_size); GD_S(3, " kbytes  *num=");
+  GD_U(3, *sh_num); GD_S(3, " *size="); GD_F64(3, *sh_size); GD_S(3, "\n");
 }
 
 int share_insert(void *data, void *user_data)
@@ -343,7 +336,7 @@ int share_insert(void *data, void *user_data)
 Gnut_List * share_copy_list(Gnut_List *l)
 {
   Gnut_List *nl=0;
-  gd_s(3, "share_copy_list entering\n");
+  GD_S(3, "share_copy_list entering\n");
   
   gnut_list_foreach(l,share_insert,&nl);
   
@@ -355,13 +348,13 @@ int share_delete(void * data, void * user_data)
 {
   share_item *si;
 
-  gd_s(3, "share_delete entering\n");
+  GD_S(3, "share_delete entering\n");
 
   si = data;
   fre_str(&(si->path), 163);
   fre_str(&(si->lc_path), 164);
   fre_str(&(si->fpath), 165);
-  gd_s(3,"share_delete returning\n");
+  GD_S(3, "share_delete returning\n");
   return 0;
 }
 
@@ -370,12 +363,12 @@ int share_delete(void * data, void * user_data)
  * clears the share list specified by l */
 int share_clear_list(Gnut_List *l)
 {
-  gd_s(3,"share_clear_list entering\n");
+  GD_S(3, "share_clear_list entering\n");
 
   gnut_list_foreach(l, share_delete, 0);
   gnut_list_fre(l);
 
-  gd_s(3,"share_clear_list returning\n");
+  GD_S(3, "share_clear_list returning\n");
   return 0;
 }
   
@@ -385,25 +378,25 @@ int share_clear_list(Gnut_List *l)
  * clears out the list of shares */
 int share_clear()
 {
-  gd_s(3,"share_clear entering\n");
+  GD_S(3, "share_clear entering\n");
   gnut_list_foreach(share_root, share_delete, 0);
   gnut_list_fre(share_root);
   share_root = 0;
   g_share_num = 0;
   g_share_size = 0;
-  gd_s(3,"share_clear returning\n");
+  GD_S(3, "share_clear returning\n");
   return 0;
 }
 
 /* share_item *share_find(int ref)
  *
  * locates the share indexed by ref and returns it */
-share_item *share_find(uint32 ref) /* 0.4.27.c11 */
+share_item *share_find(uint32 ref)
 {
   Gnut_List *ltmp;
   share_item *si,*si2;
 
-  gd_s(3,"share_find entering\n");
+  GD_S(3, "share_find entering\n");
 
   for (ltmp=share_root; ltmp; ltmp=gnut_list_next(ltmp)) {
     si = ltmp->data;
@@ -416,132 +409,41 @@ share_item *share_find(uint32 ref) /* 0.4.27.c11 */
       si2->path = ystdup(si->path, 427);
       si2->lc_path = ystdup(si->lc_path, 428);
       si2->fpath = ystdup(si->fpath, 429);
-      gd_s(3, "share_find returning si2=");
-      gd_p(3, si2);
-      gd_s(3, "\n");
+      GD_S(3, "share_find returning si2="); GD_P(3, si2); GD_S(3, "\n");
       return si2;
     }
   }
-  gd_s(3,"share_find returning failure\n");
+  GD_S(3, "share_find returning failure\n");
   return 0;
 }
 
-#ifdef WIN32
-void log_win32_error(char *szFunctionName)
-{
-  LPVOID lpMsgBuf;
-  FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM |
-				FORMAT_MESSAGE_IGNORE_INSERTS,
-				0, GetLastError(), MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), 
-				(LPSTR) &lpMsgBuf, 0, 0);
-    
-  gd_s(3, "log_win32_error "); gd_s(3, szFunctionName); gd_s(3, ": ");
-  gd_s(3, (LPSTR) lpMsgBuf); gd_s(3, "\n");
-  LocalFree(lpMsgBuf);
-}
+/* Scanning code should work with normal find (not GNU find)
 
-/* copies into pszFilePath a version of szPath that is ensured to end
- * in a "\*.*" */
+dogcow@redback.com sent this:
 
-void cleanup_directory_path(char *szPath, char *pszFilePath,
-							int *pnFilePathLength)
-{
-  strcpy(pszFilePath, szPath);
-  *pnFilePathLength = strlen(szPath);
-  
-  /* TODO: if szPath ends in \*.* bail out */
-  
-  if (L'\\' != *( pszFilePath + *pnFilePathLength-1)) {
-    strcat(pszFilePath, TEXT("\\"));
-  }
-    
-  strcat(pszFilePath, TEXT("*.*"));
-}
+in src/share.c, starting around line 549...
 
-int share_path_len=0;
+      sprintf(leafname, "find %s -follow \\( -type f "
+              "! -name \"*.gnut\" -o -type l \\) "
+        "-print"); 
 
-int share_scan_dir_win32(char *szPath)
-{
-  char szFilePath[OUR_MAX_PATH+1];
-  char szRootFilePath[OUR_MAX_PATH+1];
-  ULONG  nFilePathLength;
-  DWORD  dwFileAttributes;
-  HANDLE  hSearch;
-  WIN32_FIND_DATA  Win32FindData;
-  share_item *si;
-  
-  dwFileAttributes = GetFileAttributesA( szPath );
-  if (0xFFFFFFFF == dwFileAttributes)
-    return 0;
-  
-  if (dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) {
-    cleanup_directory_path(szPath, szRootFilePath, &nFilePathLength);
-  }
-  
-  hSearch = FindFirstFile (szRootFilePath, &Win32FindData);
-  if (INVALID_HANDLE_VALUE == hSearch) {
-    log_win32_error("share_scan.dir_win32");
-    return 0;
-  }
-  
-  /* Make sure we ignore the "." and ".." files */
-  while (!strcmp(Win32FindData.cFileName, TEXT("."))
-    || !strcmp(Win32FindData.cFileName, TEXT("..")))
-  {
-    if (!FindNextFile(hSearch, &Win32FindData)) {
-      log_win32_error("share_scan.dir_win32");
-      FindClose(hSearch);
-      return 0;
-    }
-  }
-  
-  do {
-    strcpy(szFilePath, szPath);
-    nFilePathLength = strlen( szFilePath );
-    if (L'\\' != *(szFilePath + nFilePathLength - 1)) {
-      strcat(szFilePath, TEXT("\\"));
-    }
-    strcat(szFilePath, Win32FindData.cFileName);
-    
-    if (Win32FindData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) {
-      gd_s(3, "share_scan.dir_win32: scanning %s \n",szFilePath);
-      share_scan_dir_win32(szFilePath);
-    } else {
-      si = (share_item *)ymaloc(sizeof(share_item), 292);
-      si->size = Win32FindData.nFileSizeLow;
+    (without the printf jazz)
 
-      g_share_size += ((double) (si->size));
-      si->ref = ++g_share_num;
-      si->path = ystdup(szFilePath+share_path_len+1, 430);
-      gnut_strdelimit(si->path, path_slash, '/');
-      si->lc_path = ystdup(szFilePath+share_path_len+1, 431);
-      gnut_strdelimit(si->lc_path, path_slash, '/');
-      si->fpath = ystdup(szFilePath, 432);
-      
-      /* lowercase the lc_path */
-      make_lc(si->lc_path);
-      make_7bit(si->lc_path);
+      [all the other popen and setup goo]
 
-      pthread_mutex_lock(&share_mutex);
-      share_root = gnut_list_prepend(share_root, si);
-      pthread_mutex_unlock(&share_mutex);
-    }
-  } while (FindNextFile(hSearch, &Win32FindData));
+  while (fgets(pathname, OUR_MAX_PATH, findfp)) {
+    slashloc = rindex(pathname, '/'); 
+    leafname = (slashloc ? slashloc + 1 : pathname);
+
+    [continue on as normal, with the lnend = leafname stuff]
+ */
 
-  FindClose(hSearch);
-  return 0;
-}
-#endif
 
 /* int scan.dir(char *path)
  *
  * adds the contents of path to the share_root list */
 int share_scan_dir(char *path, int verbose)
 {
-#ifdef WIN32
-  share_path_len = strlen(path);
-  return share_scan_dir_win32(path);
-#else
   FILE *findfp;
   char *leafname;
   char *lnend;
@@ -551,7 +453,7 @@ int share_scan_dir(char *path, int verbose)
   share_item *si;
   int sl;
 
-  gd_s(3, "share_scan.dir entering\n");
+  GD_S(3, "share_scan.dir entering\n");
 
   sl = conf_get_int("follow_symlinks");
 
@@ -615,18 +517,18 @@ int share_scan_dir(char *path, int verbose)
     sprintf(leafname, "echo -n ''");
   }
 
-  gd_s(1, "share_scan.dir about to run: \""); gd_s(1, leafname);
-  gd_s(1, "\" \n");
+  GD_S(1, "share_scan.dir about to run: \""); GD_S(1, leafname);
+  GD_S(1, "\" \n");
 
   /* Open pipe from command */
   findfp = popen(leafname, "r");
   if (findfp == 0) {
-    gd_s(1, "share_scan.dir pipe to find returned error: "); gd_i(1, errno);
-    gd_s(1, "\n");
+    GD_S(1, "share_scan.dir pipe to find returned error: "); GD_I(1, errno);
+    GD_S(1, "\n");
     pthread_mutex_unlock(&share_mutex);
     return -1;
   }
-  gd_s(1, "share_scan.dir retrieving results from find...\n");
+  GD_S(1, "share_scan.dir retrieving results from find...\n");
   while (fgets(leafname, OUR_MAX_PATH, findfp)) {
     if (fgets(pathname, OUR_MAX_PATH, findfp)) {
       /* remove trailing eol */
@@ -649,14 +551,14 @@ int share_scan_dir(char *path, int verbose)
 
 	si = (share_item *) ymaloc(sizeof(share_item), 295);
 
-	si->size = (uint32) (st.st_size); /* 0.4.27.c11 %%% man 2 stat does not specify if st_size is signed or unsigned */
+	si->size = (uint32) (st.st_size); /* %%% man 2 stat does not specify if st_size is signed or unsigned */
 	si->mtime = st.st_mtime;
 	g_share_size += ((double) (st.st_size));
 	g_share_num++;
 	si->ref = g_share_num;
 
 	si->path = ystdup(leafname, 433); /* returned in query reply */
-	si->lc_path = ystdup(leafname, 434); /* used in query match */
+	si->lc_path = ystdup(pathname, 434); /* used in query match */
 	si->fpath = ystdup(pathname, 435); /* real fullpath */
 
         /* lowercase the lc_path */
@@ -675,7 +577,6 @@ int share_scan_dir(char *path, int verbose)
   pthread_mutex_unlock(&share_mutex);
   
   return 0;
-#endif
 }
 
 /* my test implementation of search result caching, what I've got is a
@@ -691,13 +592,13 @@ int share_scan_dir(char *path, int verbose)
 uchar share_cache_hash_func(void *d)
 {
   share_query_results *ss;
-  uint32 crc; /* 0.4.27.c09 */
+  uint32 crc;
 
   ss = d;
-  crc = crc32_string(ss->query); /* 0.4.27.c09 */
-  crc ^= (crc >> 16); /* 0.4.27.c09 */
-  crc ^= (crc >> 8); /* 0.4.27.c09 */
-  return (crc & 0xff); /* 0.4.27.c09 */
+  crc = crc32_string(ss->query);
+  crc ^= (crc >> 16);
+  crc ^= (crc >> 8);
+  return (crc & 0xff);
 }
 
 int share_cache_compare_func(void *a, void *b)
@@ -715,46 +616,9 @@ int share_init()
   have_find = trycmd("find --version", "GNU fi");
   have_perl = trycmd("perl -we 'print \"Perl rules!\\n\";'", "rules");
 
-#if 0  /* 0.4.27.c17 */
-  share_cache_hash = gnut_hash_new(share_cache_hash_func,
-				   share_cache_compare_func);
-  share_cache_queue = gnut_queue_new(511);
-#endif
   return 0;
 }
 
-#if 0  /* 0.4.27.c17 */
-Gnut_List *share_cache_run_search(char *token, int max)
-{
-  share_query_results ss;
-  share_query_results *ss2;
-  Gnut_List *gl;
-  int n_reslts;
-  
-  ss.query = token;
-  
-  dqi(0x015b);
-  ss2 = gnut_hash_find(share_cache_hash, &ss);
-  if (ss2) {
-    /* we hit a query in our cache, so we should return a copy of this
-     * list, and possibly move this query up in the queue */
-    return share_copy_list(ss2->results);
-  }
-  /* OK, we didn't have it in the hash, which means we need to
-   * run the query, and then insert it into the cache, then return a copy */
-  
-  gl = share_search(token, max, &n_reslts);
-  ss2 = (share_query_results *) ymaloc(sizeof(share_query_results), 287);
-  ss2->query = ystdup(token, 423);
-  ss2->results = share_copy_list(gl);
-
-  gnut_hash_insert(share_cache_hash, ss2, 494);
-  gnut_queue_insert(share_cache_queue, ss2, 365);
-
-  return gl;
-}
-#endif
-
 /* take two share lists, and make a new list, not to exceed max num
  * which is the logical AND of the two inputs. */
 Gnut_List *share_results_combine(Gnut_List *l1, Gnut_List *l2,int max)
@@ -850,7 +714,6 @@ Gnut_List *share_results_combine(Gnut_List *l1, Gnut_List *l2,int max)
    memory overhead for this search-match cache is therefore a little
    less than 300K bytes. For more typical Gnutella users (sharing
    maybe 100 files) the overhead would be about 20K.
-
  */
 
 /*
@@ -861,11 +724,7 @@ Gnut_List *share_results_combine(Gnut_List *l1, Gnut_List *l2,int max)
 
 #define SHARE_HASH_N 500
 
-#ifndef PTHREADS_DRAFT4
-pthread_mutex_t share_hash_mutex=PTHREAD_MUTEX_INITIALIZER;
-#else
-pthread_mutex_t share_hash_mutex;
-#endif
+/* 0.4.28.c01 Moved mutex to gnut.c */
 
 share_item **si_vector = 0;
 share_item **siv2 = (share_item **) 1;
@@ -985,7 +844,7 @@ void share_hash_init()
 
   i = gnut_list_size(share_root);
   if (gc_debug_opts & 128) { /* tagok */
-    printf("list %d share_num %u\n", i, g_share_num); /* 0.4.27.c11 */
+    printf("list %d share_num %u\n", i, g_share_num);
   }
   /*   share_num = i; */
 
@@ -1215,14 +1074,14 @@ unsigned int new_search_term(char* term, match_bitvector *mv,
 {
   unsigned int n_matches;
   share_item* si;
-  uint32 i; /* 0.4.27.c11 */
+  uint32 i;
 
   n_matches = 0;
 
   dqi(0x00f0);
 
   if (gc_debug_opts & 128) { /* tagok */
-    printf("share_num %u\n", g_share_num); /* 0.4.27.c11 */
+    printf("share_num %u\n", g_share_num);
   }
 
   memset(scratch, 0, matchvectorsize * sizeof(uint32));
@@ -1449,7 +1308,7 @@ int share_hash_clear_list(Gnut_List *l)
 
   l2 = l;
 
-  gd_s(3,"share_hash_clear_list entering\n");
+  GD_S(3,"share_hash_clear_list entering\n");
 
   /* can't use gnut_list_fre, because I don't want to free the list
    * data items.  (they're the share_items in si_vector)  */
@@ -1468,6 +1327,6 @@ int share_hash_clear_list(Gnut_List *l)
     pthread_mutex_unlock(&share_hash_mutex);
   }
 
-  gd_s(3,"share_hash_clear_list returning\n");
+  GD_S(3,"share_hash_clear_list returning\n");
   return 0;
 }
