@@ -12,16 +12,16 @@
 typedef struct {
   int sock;
   uint8 ip[4];
-  uint16 port;
+  uint16 gt_port;
   int fsock;
   char *fname;
   int type;  /* indicates whether an upload, or download, or push */
   uint32 gt_bytes;
-  uint32 total; /* 0.4.27.c11 */
+  uint32 gt_total;
   uint32 rate_bytes;
   time_t rate_incr_time;
   uint32 rate_limit;
-  int state;  /* states are the same as for connections */
+  int gt_state;  /* states are the same as for connections */
   int substate;
   int32 gt_uindex;  /* for ui purposes... */
   pthread_t tid;
